﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using WpfApp21Point.Classes;
using WpfApp21Point.Models;
using static System.Net.Mime.MediaTypeNames;

namespace WpfApp21Point
{
    /// <summary>
    /// Логика взаимодействия для DeskPage.xaml
    /// </summary>
    public partial class DeskPage : Page
    {
        PackOfCards packOfCards = new();
        List<Card> cardsUser = new();
        List<Card> cardsComp = new();
        bool computerPlayOn = true;
        GameParty gameParty = new();
        Opponent opponent;

        public DeskPage()
        {
            InitializeComponent();
            ControllPackOfCards();
            SetOpponent();
        }

        void SetOpponent()
        {
            List<Opponent> opponents = GlobalData.DbConnect.Opponents.ToList();
            opponent = opponents[new Random().Next(opponents.Count)];
            string filePath = Environment.CurrentDirectory + opponent.PathAvatar;
            ImageOpponent.Source = new BitmapImage(new Uri(filePath));
            NameOpponent.Text = opponent.Name;
            WinsOpponent.Text = "Побед: " + opponent.CountWin.ToString();
        }

        void ControllPackOfCards()
        {
            if(packOfCards.FullPack.Count < 0)
            {
                PackOfCards_Button.Background = new SolidColorBrush(Color.FromArgb(0, 0xF0, 0x00, 0xFF));
                PackOfCards_Button.Visibility = Visibility.Collapsed;
            }
        }

        void AddCard(List<Card> cards)
        {
            Card fellCard = packOfCards.FullPack.First();
            cards.Add(fellCard);
            packOfCards.FullPack.RemoveAt(0);
            if(cards == cardsUser) 
            {
                cards.Last().IsFaceDown = false;
                string text = "Очки: " + cards.Last().Value.Value;
                CardUserControl cardView = new(fellCard, text);
                UserDesk.Children.Add(cardView);
                int score = SumCard(cardsUser);
                Score.Text = "Ваш счёт: " + score.ToString();
            }
            if(cards == cardsComp) 
            {
                CardUserControl cardView = new(fellCard, "");
                ComputerDesk.Children.Add(cardView);
            }
            ControllPackOfCards();
        }

        int SumCard(List<Card> cards)
        {
            return cards.Sum(c => c.Value.Value);
        }

        private void TakeCardButton_Click(object sender, RoutedEventArgs e)
        {
            AddCard(cardsUser);            
            ComputersMove();
            int scoreUser = SumCard(cardsUser);
            if (scoreUser >= 21)
            {
                while (computerPlayOn)
                {
                    ComputersMove();
                }
                if(scoreUser == 21)
                {
                    MessageBox.Show("Вы набрали нужное количество очков!");
                }
                else
                {
                    MessageBox.Show("Вы проиграли! Очков больше, чем 21.");
                }
                ControllStopGame();
            }
        }

        private void ComputersMove()
        {
            if (!computerPlayOn)
            {
                return;
            }
            int maxNeedValue = 21 - SumCard(cardsComp);
            int countFavorableOutcomes = 0;
            List<CardValue> cardValues = GlobalData.DbConnect.CardValues.Where(v => v.Value <= maxNeedValue).ToList();
            foreach (CardValue cardValue in cardValues)
            {
                countFavorableOutcomes += 4 - cardsComp.Where(c => c.Value == cardValue).Count();
            }
            double probability = (double)countFavorableOutcomes/ (double)packOfCards.FullPack.Count();
            //double roundProbality = Math.Round(probability, 2);
            if(probability < opponent.MinimalProbability)
            {
                computerPlayOn = false;
            }
            if (computerPlayOn)
            {
                AddCard(cardsComp);
            }
        }

        private void FlipPachComp()
        {
            PackOfCards_Button.IsEnabled = false;
            string filePath = Environment.CurrentDirectory + @"//Picture//Cards//Рубашка.gif";
            BitmapImage bitmap = new BitmapImage(new Uri(filePath));
            var brush = new ImageBrush();
            brush.ImageSource = bitmap;
            PackOfCards_Button.Background = brush;
            ComputerDesk.Children.Clear();
            //TextBlock textBlock = new TextBlock();
            //textBlock.Text = "Количество очков: " + cardsComp.Sum(c => c.Value.Value).ToString();
            //textBlock.Foreground = new SolidColorBrush(Colors.White);
            //ComputerDesk.Children.Add(textBlock);
            foreach (Card card in cardsComp)
            {
                card.IsFaceDown = false;
                string text = "Очки: " + card.Value.Value;
                CardUserControl cardView = new(card, text);
                ComputerDesk.Children.Add(cardView);
            }
        }

        private void ControllStopGame()
        {
            StopGameButton.IsEnabled = false;
            FlipPachComp();
            int scoreUser = SumCard(cardsUser);
            int scoreComp = SumCard(cardsComp);
            if (scoreUser > 21 || scoreUser == scoreComp)
            {
                gameParty.IsWin = false;
            }
            else if (scoreComp == 21)
            {
                opponent.CountWin = opponent.CountWin + 1;
                gameParty.IsWin = false;
            }
            else if(scoreUser == 21)
            {
                gameParty.IsWin = true;
            }
            else if(scoreUser < 21)
            {
                if (scoreUser < scoreComp && scoreComp < 21)
                {
                    opponent.CountWin = opponent.CountWin + 1;
                    gameParty.IsWin = false;
                }
                else
                {
                    gameParty.IsWin = true;
                }
            }
            if(GlobalData.User != null)
            {
                gameParty.User = GlobalData.User;
                gameParty.UserId = GlobalData.User.Id;
            }
            if(!gameParty.IsWin)
            {
                GlobalData.DbConnect.Opponents.Update(opponent);
                GlobalData.DbConnect.SaveChanges();
            }
            gameParty.Date = DateTime.Now;
            gameParty.AmountOfPointsUser = scoreUser;
            gameParty.AmountOfPointsOpponent = scoreComp;

            ResultGameWindow resultGameWindow = new(gameParty);            
            resultGameWindow.Show();
        }

        private void StopGameButton_Click(object sender, RoutedEventArgs e)
        {
            while (computerPlayOn)
            {
                ComputersMove();
            }
            ControllStopGame();
        }
    }
}
