﻿using Microsoft.EntityFrameworkCore;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;
using WpfApp21Point.Classes;
using WpfApp21Point.Models;

namespace WpfApp21Point
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        List<User> ratingUser;
        public MainWindow()
        {
            InitializeComponent();
            GlobalData.MainFrame = DeskFrame;
            GlobalData.MainFrame.Navigate(new StartGamePage());
            CheckAuth();
            ratingUser = GlobalData.DbConnect.Users
                .Include(x => x.GameParties)
                .Where(x=> x.GameParties.Count > 0)
                .ToList().OrderByDescending(x => x.CountOfWin).Take(10).ToList();
            RatingTable.ItemsSource = ratingUser;
            StartTimer();
        }

        private void StartTimer()
        {
            DispatcherTimer dispatcherTimer = new System.Windows.Threading.DispatcherTimer();
            dispatcherTimer.Tick += new EventHandler(dispatcherTimer_Tick);
            dispatcherTimer.Interval = TimeSpan.FromSeconds(10);
            dispatcherTimer.Start();
        }

        private void dispatcherTimer_Tick(object? sender, EventArgs e)
        {
            ratingUser = GlobalData.DbConnect.Users
                .Include(x => x.GameParties)
                .Where(x => x.GameParties.Count > 0)
                .ToList().OrderByDescending(x => x.CountOfWin).Take(10).ToList();
            RatingTable.ItemsSource = ratingUser;
            StartTimer();
        }

        private void CheckAuth()
        {
            if (GlobalData.User != null)
            {
                AuthUser_Panel.Visibility = Visibility.Collapsed;
                UserInfo_Panel.Visibility = Visibility.Visible;
                UserName_TextBlock.Text = "Игрок: " + GlobalData.User.UserName;
                ExitButton.Visibility = Visibility.Visible;
            }
            else
            {
                AuthUser_Panel.Visibility = Visibility.Visible;
                UserInfo_Panel.Visibility = Visibility.Collapsed;
                UserName_TextBlock.Text = "";
                ExitButton.Visibility = Visibility.Collapsed;
            }
        }

        private void Auth_Click(object sender, RoutedEventArgs e)
        {
            AuthWindow authWindow = new("Войти");
            authWindow.ShowDialog();
            CheckAuth();
        }

        private void CreateUser_Click(object sender, RoutedEventArgs e)
        {
            AuthWindow authWindow = new("Создать");
            authWindow.ShowDialog();
            CheckAuth();
        }

        private void RulesButton_Click(object sender, RoutedEventArgs e)
        {
            RulesWindow rulesWindow = new();
            rulesWindow.Show();
        }

        private void UserRaiting_Click(object sender, RoutedEventArgs e)
        {
            UserPartiesWindow userPartiesWindow = new();
            userPartiesWindow.Show();
        }

        private void ExitButton_Click(object sender, RoutedEventArgs e)
        {
            GlobalData.User = null;
            CheckAuth();
        }
    }
}